#![cfg_attr(
    all(not(debug_assertions), target_os = "windows"),
    windows_subsystem = "windows"
)]

#[cfg(windows)]
use {
    std::fs::File,
    std::os::windows::io::FromRawHandle,
    std::os::raw::c_void,
    gag::Redirect,
    windows::Win32::System::Console::{
        AllocConsole,
        AttachConsole,
        GetStdHandle,
        STD_OUTPUT_HANDLE,
        STD_ERROR_HANDLE,
    }
};

mod app;
mod cli;
mod config;
mod credentials;
mod errors;
mod clientinfo;
mod ipc;
mod state;
mod server;
mod tray;

use crate::errors::ErrorPopup;

use std::io::Write;
fn main() {
    let args = cli::parser().get_matches();
    let help = matches!(args.get_one::<bool>("help"), Some(true));

    // This is the only case that doesn't need a console
    if let None | Some(("run", _)) = args.subcommand() {
        if !help {
            app::run().error_popup("Creddy failed to start");
            return;
        }
    }

    // on Windows, need to do the whole allocate-a-console thing
    #[cfg(windows)]
    attach_console();
    // let (out, err) = setup_console();
    println!("Testing stdout");
    // writeln!(&mut out, "Testing allocated file");

    if help {
        // if we can't print help, we can't print an error, so just panic
        dbg!(args.get_one::<bool>("help"));
        cli::parser().print_help().unwrap();
        std::thread::sleep(std::time::Duration::from_secs(3));
        return;
    }

    let res = match args.subcommand() {
        Some(("show", m)) => cli::show(m),
        Some(("exec", m)) => cli::exec(m),
        // clap ensures that subcommand is either None or run/show/exec
        _ => unreachable!(),
    };

    if let Err(e) = res {
        eprintln!("Error: {e}");
    }
}


fn attach_console() {
    unsafe { AttachConsole(u32::MAX); }
}



fn setup_console() -> (Redirect<File>, Redirect<File>) {
    let (mut stdout, stderr) = unsafe {
        AllocConsole();
        // if we can't get handles to stdout/err, we can't display these errors,
        // so there's no point in doing anything other than panicking
        let stdout_handle = GetStdHandle(STD_OUTPUT_HANDLE).unwrap();
        let stderr_handle = GetStdHandle(STD_ERROR_HANDLE).unwrap();

        let mut f = File::create("C:\\Users\\Joe\\Downloads\\debug.txt").unwrap();
        writeln!(&mut f, "{stdout_handle:?}\n{:?}", stdout_handle.0 as *mut c_void).unwrap();

        (
            File::from_raw_handle(stdout_handle.0 as *mut c_void),
            File::from_raw_handle(stderr_handle.0 as *mut c_void),
        )
    };
    
    writeln!(&mut stdout, "Testing stdout before redirect")
        .map_err(|e| log_err(e))
        .unwrap();

    (
        Redirect::stdout(stdout).unwrap(),
        Redirect::stderr(stderr).unwrap()
    )
}


fn log_err(e: impl std::error::Error) {
    let mut f = File::create("C:\\Users\\Joe\\Downloads\\log.txt").unwrap();
    writeln!(&mut f, "{e}").unwrap();
}
